#include "gpib_driver.h"
// Note this was written by looking at all sorts of sources, I think it
// will work for most National Instruments boards and for all boards using
// the 7210 GPIB controller chip which is now made by many companies
// We developed the library for linux kernels 2.2.xx but it doesn't do anything
// fancy and should works else where.  The base address and IO ports will be your
// own concern if you are lucky enough NOT to have a NI or ICS board.
// These boards have their registers spaced by 0x400 bytes (defined in gpib_driver
// by GPIB_REG_STEP) if you have something remotely new or sane try GPIB_REG_STEP=1
//
// We got the information to write this from many sources including various 7210 
// makers and various board makers, A company making these things called "Computer Boards"
// had possibly the most useful information, ICS were possibly the rudest and least
// helpful people I dealt with, providing no information and asking us to sign away
// all our rights for it.  Hmmm we weren't best pleased.  Shame really as we were 
// supporting ICS boards at the time.


// deal with the legal addresses
// we put this in because the NI and ICS board cover a vast amount
// of IO space (although sparsely) and we wanted not to trash other
// boards by mistake.  Not really a problem as ISA is dying out
#ifdef GPIB_ICS
//set this for national instruments boards and ics boards
int GPIB_LEGAL_ADDRS[]={0x02e1,0x22e1,0x42e1,0x62e1};
int GPIB_NO_LEGAL_ADDRS=4;
#else
//int GPIB_LEGAL_ADDRS={Enter your own numbers here-sorry matt};
//int GPIB_NO_LEGAL_ADDRS=enter then number of legal numbers here see the .c file for the code;
#endif

GPIB_baseaddress GPIB_IO_base_address=-1;

int	GPIB_EOT_mode=GPIB_EOT_NEW_LINE;
char	GPIB_board_gpib_address;

// a wrapper for the reuired pausing function
// usleep(1) is not reliable, you often get >10us but so what, if you
// want to be precise use outb(1,0x80); to get 1us
void	GPIB_pause(void){
	// keep if you need it matt nov 2000
	//usleep(1);
	}
#ifdef MODULE
#define SLEEP(a) udelay(a)
#endif
#ifndef MODULE
#define SLEEP(a) usleep(a)
#endif

int GPIB_initboard(GPIB_baseaddress ba,int board_home){
int i;
	GPIB_IO_base_address=ba;	
	GPIB_board_gpib_address=board_home;
	for(i=0;i<GPIB_NO_LEGAL_ADDRS;i++){
		if(GPIB_IO_base_address==GPIB_LEGAL_ADDRS[i])break;
		}
	if(i>=GPIB_NO_LEGAL_ADDRS)
		return GPIB_ERROR_ILLEGAL_ADDRESS;
			
	// Reset the 7210 chip
	GPIB_BYTE_OUT(GPIB_IO_base_address,GPIB_REG_AUX_MODE,GPIB_CMD_CHIP_RESET); 
	GPIB_BYTE_OUT(GPIB_IO_base_address,GPIB_REG_AUX_MODE,GPIB_CMD_EXEC_PON);

	// Clear the gpib bus IFC line
	GPIB_BYTE_OUT(GPIB_IO_base_address,GPIB_REG_AUX_MODE,GPIB_CMD_CLEAR_IFC);  
	
	// Clear the register by reading it
	GPIB_BYTE_IN(GPIB_IO_base_address,GPIB_REG_CMD_PASS_THRU);

	// Disable any interrupts (shouldn't be any but better safe than sorry)
	GPIB_BYTE_OUT(GPIB_IO_base_address,GPIB_REG_INT_MASK1,0);
	GPIB_BYTE_OUT(GPIB_IO_base_address,GPIB_REG_INT_MASK2,0);

	// Read to clear the device
	GPIB_BYTE_IN(GPIB_IO_base_address,GPIB_REG_INT_STAT1);
	GPIB_BYTE_IN(GPIB_IO_base_address,GPIB_REG_INT_STAT2);

	// Set up for polling
	GPIB_BYTE_OUT(GPIB_IO_base_address,GPIB_REG_SER_POLL_MODE,0);

	// Set the clock speed 
	GPIB_BYTE_OUT(GPIB_IO_base_address,GPIB_REG_AUX_MODE, 10|GPIB_REG_ICR);
	GPIB_BYTE_OUT(GPIB_IO_base_address,GPIB_REG_AUX_MODE,  GPIB_REG_AUXRB);
	
	// Init the AUX E register (don't know what this means..)
	GPIB_BYTE_OUT(GPIB_IO_base_address,GPIB_REG_AUX_MODE,GPIB_REG_AUXRE);	 

	// Enable parallel polling
	GPIB_BYTE_OUT(GPIB_IO_base_address,GPIB_REG_AUX_MODE,  GPIB_REG_PPR);

	// Set GPIB bus address of the GPIB board itself 
	GPIB_BYTE_OUT(GPIB_IO_base_address,GPIB_REG_ADR,GPIB_board_gpib_address);		  

	// disable the secondary address
	GPIB_BYTE_OUT(GPIB_IO_base_address,GPIB_REG_ADR_MODE, GPIB_BIT_TR0 | GPIB_BIT_TR1 | GPIB_BIT_ADR_MODE1);
	GPIB_BYTE_OUT(GPIB_IO_base_address,GPIB_REG_ADR,GPIB_BIT_SAD_SELECT | GPIB_BIT_NOT_TALK | GPIB_BIT_NOT_LISTEN);

	GPIB_BYTE_OUT(GPIB_IO_base_address,GPIB_REG_AUX_MODE, GPIB_CMD_EXEC_PON);

	GPIB_IFC_strobe();
	if (GPIB_active()!=GPIB_OK){
			return GPIB_ERROR;
			}

	// bug fix for the 7210 chip, this routine fixes it (from computer boards..)
	if(GPIB_init488chip())return GPIB_ERROR;
	// enable remote control
	if(GPIB_RENLine(TRUE))return GPIB_ERROR;

		
	return GPIB_OK;
	}

// Now read up to N bytes until the GPIB_BIT_END_RCVD is found.
// N bytes are read and GPIB_BIT_END_RCVD is not set report an error
// THIS matters---matt oct 2000
// note max transfer size 1 integer size, change to long if it bothers you

int GPIB_readdata(char *buff, int N){
int	count=0,tries=0;
char	status;
	// enforce a slight delay, consecutive messages
	// can collide otherwise
	if (N==0){
		return 0;
		}
	/* moved to gpib usr
	/*if (GPIB_standby()!=GPIB_OK)
		return GPIB_ERROR;*/
	do{	
		status=GPIB_BYTE_IN(GPIB_IO_base_address,GPIB_REG_INT_STAT1);
		// test for data available...
		if ((status & GPIB_BIT_DATA_IN)!=0){
			buff[count++]=GPIB_BYTE_IN(GPIB_IO_base_address,GPIB_REG_DATA_IN);
			tries=0;
			}
		else{  // actually may want a proper timed pause here not a usleep()
			GPIB_pause();
			}
	}while((count<N)&&((status & GPIB_BIT_END_RCVD)==0)&&((status&GPIB_BIT_ERR)==0)&&(tries++<GPIB_MAX_TRIES));
	//printk("GPIB_DRIVER: tries=%d\n",tries);
	if(tries>=GPIB_MAX_TRIES) return GPIB_ERROR_TIMEOUT;

	//if((count==N)&&((GPIB_BYTE_IN(GPIB_IO_base_address,GPIB_REG_INT_STAT1)&&GPIB_BIT_DATA_IN)!=0)){
	if((count==N)){
		// this is a software error, the remaining bytes MUST be read or the board will require reseting
		return GPIB_ERROR_REMAINING_BYTES;
		}

	return count;
	}

int GPIB_writedata(char *buff, int N){
int 	count,tries=0;
	// enforce a slight delay, consecutive messages
	// can collide otherwise
	GPIB_pause();
	if (N==0){
		return GPIB_OK;
		}
	if (GPIB_standby()!=GPIB_OK){
		return GPIB_ERROR;
		}
	//printk("GPIB_DRIVER:");
	for(count=0;count<N;count++,tries=0){
		GPIB_BYTE_OUT(GPIB_IO_base_address,GPIB_REG_CMD_DATA_OUT,buff[count]);
		//printk("%c(%x)",buff[count],(int)buff[count]);
		while(((GPIB_BYTE_IN(GPIB_IO_base_address,GPIB_REG_INT_STAT1)&GPIB_BIT_DATA_OUT)==0)&&(tries++<GPIB_MAX_TRIES)){
			GPIB_pause();
			}
		if(tries==GPIB_MAX_TRIES) return GPIB_ERROR_TIMEOUT;
		}
	//printk("\n");
	GPIB_BYTE_OUT(GPIB_IO_base_address,GPIB_REG_AUX_MODE,GPIB_CMD_SEND_EOI);
	GPIB_pause();
	return count;
	}

// changed to send a single byte matt oct 2000
int GPIB_sendcommand(char cmd){
int wait=0;
	if((GPIB_BYTE_IN(GPIB_IO_base_address,GPIB_REG_ADR_STAT) & GPIB_BIT_CIC)==0)
		return GPIB_ERROR;

	// If standby, become active controller 
	if(GPIB_active()!=GPIB_OK)
		return GPIB_ERROR;

	GPIB_BYTE_OUT(GPIB_IO_base_address,GPIB_REG_CMD_DATA_OUT,cmd);
	while((wait++<GPIB_MAX_WAIT)&(GPIB_BYTE_IN(GPIB_IO_base_address,GPIB_REG_INT_STAT2)&GPIB_BIT_CMD_OUT)==0){
		GPIB_pause();
		}
	if(wait>=GPIB_MAX_WAIT)return GPIB_ERROR_SENDCOM_TO;
	return GPIB_OK;
	}



int GPIB_standby(void){
int wait=0;
	// test for CIC (controller in charge) if not return an error
	if ((GPIB_BYTE_IN(GPIB_IO_base_address,GPIB_REG_ADR_STAT)&GPIB_BIT_CIC)==0){
		return GPIB_ERROR;
		}

	// if already in standby return ok
	if ((GPIB_BYTE_IN(GPIB_IO_base_address,GPIB_REG_ADR_STAT)&GPIB_BIT_NOT_ATN)!=0){
		return GPIB_OK;
		}
	// else switch to standby
	GPIB_BYTE_OUT(GPIB_IO_base_address,GPIB_REG_AUX_MODE,GPIB_CMD_GO_STDBY);

	//if the board is a talker we have to wait for it to be ready
	if((GPIB_BYTE_IN(GPIB_IO_base_address,GPIB_REG_ADR_STAT)&GPIB_BIT_TALK_ACTIVE)!=0){
		while((wait++<GPIB_MAX_WAIT)&(GPIB_BYTE_IN(GPIB_IO_base_address,GPIB_REG_INT_STAT1)&GPIB_BIT_DATA_OUT)==0){
			GPIB_pause();
			}
			if(wait>=GPIB_MAX_WAIT)return GPIB_ERROR_STANDBY_TO;
		}
	return GPIB_OK;
	}


int GPIB_active(void){
int wait=0;
	// if the board is not controller in charge return
	if ((GPIB_BYTE_IN(GPIB_IO_base_address,GPIB_REG_ADR_STAT) & GPIB_BIT_CIC) == 0)
		return GPIB_ERROR;
	// if already in standby return // (i don't get this--matt march 2000)
	if ((GPIB_BYTE_IN(GPIB_IO_base_address,GPIB_REG_ADR_STAT) & GPIB_BIT_NOT_ATN) == 0)
		return GPIB_OK;
	// take control
	GPIB_BYTE_OUT(GPIB_IO_base_address,GPIB_REG_AUX_MODE,GPIB_CMD_TAKE_CTRL_A);
	// wait for this to be swallowed
	GPIB_pause();
	while((wait++<GPIB_MAX_WAIT)&(GPIB_BYTE_IN(GPIB_IO_base_address,GPIB_REG_INT_STAT2)&GPIB_BIT_CMD_OUT)==0) GPIB_pause();
	if(wait>=GPIB_MAX_WAIT)return GPIB_ERROR_ACTIVE_TO;
	return GPIB_OK;
	}

// The following functions are not well understood by us, they appeared here and there
// in other examples and are mainly used to in setting up stuff.  If you have a specific
// GPIB function that lies outside or reading and writing these may be of use to you else
// there just here because they are used elsewhere

int GPIB_endhandshake(void){
	// request a handshake and return
	GPIB_BYTE_OUT(GPIB_IO_base_address,GPIB_REG_AUX_MODE,GPIB_CMD_FINISH_HAND);
	return GPIB_getchipstatus();
	}

int GPIB_getchipstatus(void){
	// read both interrupt registers
	// these are multipurpose and contain other information
	// ignore everything and return an error if GPIB_BIT_ERR is set
	if (GPIB_BIT_ERR&GPIB_BYTE_IN(GPIB_IO_base_address,GPIB_REG_INT_STAT1))
		return GPIB_ERROR;
 
        return GPIB_OK;
	}

// a work around a 7210 bug, helps ensure initialization by
// reading something it sends itself....
int GPIB_init488chip(void){
	char command[10];

	// Address board as both listener and talker
	// should check for errors
	GPIB_sendcommand(GPIB_MTA_CMD+GPIB_board_gpib_address);
	GPIB_sendcommand(GPIB_MLA_CMD+GPIB_board_gpib_address);

	if(GPIB_standby()!=GPIB_OK)
		return GPIB_ERROR;

	GPIB_BYTE_OUT(GPIB_IO_base_address,GPIB_REG_CMD_DATA_OUT,0x20);
	GPIB_BYTE_IN(GPIB_IO_base_address,GPIB_REG_DATA_IN);
	GPIB_BYTE_IN(GPIB_IO_base_address,GPIB_REG_INT_STAT1);		
	// Read and throw away ERR bit 

	// Unaddress the board 
	GPIB_sendcommand(GPIB_UNL_CMD);
	GPIB_sendcommand(GPIB_UNT_CMD);
	return GPIB_OK;
	}

int GPIB_IFC_strobe(void){
        // strobe the IFC line up
        GPIB_BYTE_OUT(GPIB_IO_base_address,GPIB_REG_AUX_MODE,GPIB_CMD_SET_IFC);
        // pause for at least 100us
        SLEEP(100);
        // read to clear the 7210
        GPIB_BYTE_IN(GPIB_IO_base_address,GPIB_REG_EOS);
        // strobe the IFC down
        GPIB_BYTE_OUT(GPIB_IO_base_address,GPIB_REG_AUX_MODE,GPIB_CMD_CLEAR_IFC);
        // return the status
        return GPIB_getchipstatus();
        }

int GPIB_RENLine(int RENState){
        if (RENState == TRUE){
                GPIB_BYTE_OUT(GPIB_IO_base_address,GPIB_REG_AUX_MODE, GPIB_CMD_SET_REN);
                }
        else{
                GPIB_BYTE_OUT(GPIB_IO_base_address,GPIB_REG_AUX_MODE, GPIB_CMD_CLEAR_REN);
                }
        SLEEP(100);
	return GPIB_getchipstatus();
        }

// this controls the last byte of a transmission,
// ignore it unless you care, it is set to GPIB_EOT_NONE by init
int GPIB_set_EOT(int  EOT){
        GPIB_EOT_mode==EOT;
        // this can be
        // GPIB_STOPend
        // GPIB_EOT_NEW_LINE
        // GPIB_EOT_EOI
        // GPIB_EOT_NONE
 
        // not really anything we care about, should be elsewhere
        // set how the GPIB ends its message
        if (GPIB_EOT_mode==GPIB_STOPend){
                GPIB_BYTE_OUT(GPIB_IO_base_address,GPIB_REG_AUX_MODE, GPIB_REG_AUXRA+GPIB_BIT_HOLD_ON_END);
                }
        // Stop on EOS character
        else{
                GPIB_BYTE_OUT(GPIB_IO_base_address,GPIB_REG_EOS, GPIB_EOT_mode & 0xff);
                GPIB_BYTE_OUT(GPIB_IO_base_address,GPIB_REG_AUX_MODE, GPIB_BIT_BINARY_EOS + GPIB_BIT_END_ON_RCV_EOS);
                }
        }    
#ifdef MODULE
static int GPIB_sleep(int timeout_ms){
        unsigned int timeout = timeout_ms * 10 * HZ / 100;
        do {
                current->state = TASK_INTERRUPTIBLE;
                timeout = schedule_timeout(timeout);
                }while (timeout);
        return 0;
        }
#endif
