#ifndef _GPIB_DRIVER_
#define _GPIB_DRIVER_
// to avoid name space pollution all stuff begins with GPIB_

//If your board is too slow at IO try uncommenting the following line
//#define REALLY_SLOW_IO
// defines for the driver program
#define TRUE		1
#define FALSE		0

// How many us to pause in places.
#define GPIB_PAUSE			100
// define how long to wait for various lock up cases (arbitrary number)
// require for kernel drivers so as not to lock the kernel
#define GPIB_MAX_WAIT	1000

// All Errors are -ve to distingish themselves from +ve byte counts
#define GPIB_OK				0
#define GPIB_ERROR			-1
#define GPIB_ERROR_ILLEGAL_ADDRESS	-2
#define GPIB_ERROR_REMAINING_BYTES	-3
#define GPIB_ERROR_TIMEOUT		-4
#define GPIB_ERROR_STANDBY_TO		-5
#define GPIB_ERROR_SENDCOM_TO		-6
#define GPIB_ERROR_ACTIVE_TO		-7

#define GPIB_NEW_LINE_CHAR	0xa

// the setting of MAX_TRIES is abritrary based on experience
// this gives this as a high value but it will depend on the
// speed of the computer and speed of the device and compiler optimisation
// The lecroy needs around 5 goes per byte
// The marconi SGs need around 10000 goes per byte
// This value is guessed as 5 times the max you will need make it bigger if you want
// BUT rememer that "tries" is a float and may need to be a "long" if you make this
// much bigger matt 2000
#define GPIB_MAX_TRIES 		60000


// We DO NOT use these in this library, like many defines below
// They are kept here as part of the GPIB definition in case any one else
// wants to use them
// end of transmission settings
#define GPIB_EOT_NEW_LINE 	0	// send a newline char / last byte / eoi_cmd at eot
#define GPIB_EOT_EOI		1	// send last byte / eoi cmd at eot
#define GPIB_EOT_NONE   	2	// don't do anything just last byte 
#define GPIB_STOPend   0x0100  	// Receive()( termination
				// maybe useful for partial messages....

// Misc defines

// GPIB Commands  
#define GPIB_GTL_CMD        0x001        // Go to Local 
#define GPIB_SDC_CMD        0x004        // Selected device clear 
#define GPIB_PPC_CMD        0x005        // Parallel poll configure 
#define GPIB_GET_CMD        0x008        // Group execute trigger 
#define GPIB_TCT_CMD        0x009        // Take control 
#define GPIB_LLO_CMD        0x011        // Local lockout 
#define GPIB_DCL_CMD        0x014        // Device clear 
#define GPIB_PPU_CMD        0x015        // Parallel poll unconfigure 
#define GPIB_SPE_CMD        0x018        // Serial poll enable 
#define GPIB_SPD_CMD        0x019        // Serial poll disable 
#define GPIB_MLA_CMD        0x020        // My listen address 
#define GPIB_UNL_CMD        0x03f        // Unlisten 
#define GPIB_MTA_CMD        0x040        // My talk address 
#define GPIB_UNT_CMD        0x05f        // Untalk 
#define GPIB_MSA_CMD        0x060        // My secondary address 
#define GPIB_PPE_CMD        0x060        // Parallel Poll Enable 
#define GPIB_PPD_CMD        0x070        // Parallel poll disable 

// Hardware register bit definitions  

// Bit definitions within Interrupt Status Register 1 (ISR1) 
//                        Interrupt Mask Register 1 (IMR1) 
#define GPIB_BIT_DATA_IN       0x001
#define GPIB_BIT_DATA_OUT      0x002
#define GPIB_BIT_ERR           0x004
#define GPIB_BIT_DEV_CLEAR     0x008
#define GPIB_BIT_END_RCVD      0x010
#define GPIB_BIT_DEV_TRIG      0x020
#define GPIB_BIT_ADR_PASS_THRU 0x040
#define GPIB_BIT_CMD_PASS_THRU 0x080

// Bit definitions within Interrupt Status Register 2 (ISR2)
//                        Interrupt Masm Register 2 (IMR2)  
#define GPIB_BIT_ADR_STAT_CHG  0x001
#define GPIB_BIT_REMOTE_CHG    0x002
#define GPIB_BIT_LOCKOUT_CHG   0x004
#define GPIB_BIT_CMD_OUT       0x008
#define GPIB_BIT_DMA_IN        0x010
#define GPIB_BIT_REMOTE        0x010
#define GPIB_BIT_DMA_OUT       0x020
#define GPIB_BIT_LOCKOUT       0x020
#define GPIB_BIT_SERVICE_REQ   0x040
#define GPIB_BIT_INT           0x080

// Bit definitions within Serial Poll Register (SPSR)  
#define GPIB_MASK_SP_STATUS    0x03f
#define GPIB_BIT_SP_PENDING    0x040
#define GPIB_BIT_REQ_SRVC      0x040
#define GPIB_BIT_SP_STATUS     0x080

// Bit definitions Address Status Register (ADSR) 
#define GPIB_BIT_TALK_ACTIVE   0x002
#define GPIB_BIT_LISTEN_ACTIVE 0x004
#define GPIB_BIT_NOT_ATN       0x040
#define GPIB_BIT_CIC           0x080

// Bit definitions within Address Mode Register (ADMR) 
#define GPIB_BIT_ADR_MODE1     0x001
#define GPIB_BIT_ADR_MODE2     0x002
#define GPIB_BIT_ADR_MODE3     0x003
#define GPIB_BIT_TR0           0x010
#define GPIB_BIT_TR1           0x020

// Bit definitions within Auxilliary Mode register (AUXMR) 
#define GPIB_CMD_EXEC_PON      0x000
#define GPIB_CMD_CLR_PP_FLAG   0x001
#define GPIB_CMD_CHIP_RESET    0x002
#define GPIB_CMD_FINISH_HAND   0x003
#define GPIB_CMD_TRIGGER       0x004
#define GPIB_CMD_RTRN_LOCAL    0x005
#define GPIB_CMD_SEND_EOI      0x006
#define GPIB_CMD_SET_PP_FLAG   0x009
#define GPIB_CMD_TAKE_CTRL_A   0x011
#define GPIB_CMD_TAKE_CTRL_S   0x012
#define GPIB_CMD_TAKE_CTRL_E   0x01a
#define GPIB_CMD_GO_STDBY      0x010
#define GPIB_CMD_LISTEN        0x013
#define GPIB_CMD_LISTEN_CONT   0x01b
#define GPIB_CMD_LOC_UNLISTEN  0x01c
#define GPIB_CMD_EXEC_PP       0x01d
#define GPIB_CMD_SET_IFC       0x01e
#define GPIB_CMD_CLEAR_IFC     0x016
#define GPIB_CMD_SET_REN       0x01f
#define GPIB_CMD_CLEAR_REN     0x017
#define GPIB_CMD_DIS_SYS_CTRL  0x014

#define GPIB_REG_ICR           0x020
#define GPIB_REG_PPR           0x060
#define GPIB_REG_AUXRA         0x080
#define GPIB_REG_AUXRB         0x0a0
#define GPIB_REG_AUXRE         0x0c0

// Bit definitions within Parellel Mode Register (PPR) 
#define GPIB_MASK_PP_RESPONSE  0x007
#define GPIB_BIT_PP_POLARITY   0x008
#define GPIB_BIT_PP_UNCONFIG   0x010

// Bit definitions within Auxilliary Register A (AUXRA) 
#define GPIB_BIT_NORMAL_HSHAKE   0x000
#define GPIB_BIT_HOLD_ON_ALL     0x001
#define GPIB_BIT_HOLD_ON_END     0x002
#define GPIB_BIT_CONTINUOUS      0x003
#define GPIB_BIT_END_ON_RCV_EOS  0x004
#define GPIB_BIT_TX_END_WITH_EOS 0x008
#define GPIB_BIT_BINARY_EOS      0x010

// Bit definitions within Auxilliary Register B (AUXRB) 
#define GPIB_BIT_CPT_ENABLE   0x001
#define GPIB_BIT_SEND_SP_EOI  0x002
#define GPIB_BIT_HIGH_SPEED   0x004
#define GPIB_BIT_INVERT_INT   0x008
#define GPIB_BIT_ISS          0x010

// Bit definitions within Address Register 0 (ADR0)
#define GPIB_ADR_MASK         0x01f
#define GPIB_BIT_NOT_LISTEN   0x020
#define GPIB_BIT_NOT_TALK     0x040
#define GPIB_BIT_SAD_SELECT   0x080

// Bit Definitions within hidden line status register 
#define GPIB_ATN_LINE         0x001
#define GPIB_EOI_LINE         0x002
#define GPIB_SRQ_LINE         0x004
#define GPIB_IFC_LINE         0x008
#define GPIB_REN_LINE         0x010
#define GPIB_DAV_LINE         0x020
#define GPIB_NRFD_LINE        0x040
#define GPIB_NDAC_LINE        0x080

// Bit definitions within 488.2 HS control register 
#define GPIB_BIT_TX_ENABLE    0x01
#define GPIB_BIT_RX_ENABLE    0x02
#define GPIB_BIT_ODD_BYTE     0x04
#define GPIB_BIT_HF_INT_EN    0x08
#define GPIB_BIT_CLR_SRQ_INT  0x10
#define GPIB_BIT_CLR_EOI_INT   0x20   // When RX is enable 
#define GPIB_BIT_CLR_EMPTY_INT 0x20   // When TX is enable 
#define GPIB_BIT_CLR_HF_INT   0x40
#define GPIB_BIT_SYS_CONTROL  0x80

// Bit definitions within 488.2 HS status register 
#define GPIB_BIT_FIFO_FULL    0x01
#define GPIB_BIT_HALF_FULL    0x02
#define GPIB_BIT_SRQ_INT      0x04
#define GPIB_BIT_EOI_INT      0x08
#define GPIB_BIT_TX_MSB_EMPTY 0x10
#define GPIB_BIT_RX_MSB_EMPTY 0x20
#define GPIB_BIT_TX_LSB_EMPTY 0x40
#define GPIB_BIT_RX_LSB_EMPTY 0x80

// Bit definitions within 488.2 board's REG_HS_INT_LEVEL register 
#define GPIB_BIT_RESET_7210   0x80

// ibsta bits that are set for devices 
#define GPIB_DEV_STAT_MASK 0x0e900

#define GPIB_INT_MASK1  0x0
#define GPIB_INT_MASK2  0x0

#define GPIB_ICS
#ifdef GPIB_ICS
#define GPIB_REG_STEP        0x400  
#else
#define GPIB_REG_STEP      1 
#endif

// 7210 write register offsets 
#define GPIB_REG_CMD_DATA_OUT  0*GPIB_REG_STEP
#define GPIB_REG_INT_MASK1     1*GPIB_REG_STEP
#define GPIB_REG_INT_MASK2     2*GPIB_REG_STEP
#define GPIB_REG_SER_POLL_MODE 3*GPIB_REG_STEP
#define GPIB_REG_ADR_MODE      4*GPIB_REG_STEP
#define GPIB_REG_AUX_MODE      5*GPIB_REG_STEP
#define GPIB_REG_ADR           6*GPIB_REG_STEP
#define GPIB_REG_EOS           7*GPIB_REG_STEP

// 7210 write register offsets 
#define GPIB_REG_DATA_IN       0*GPIB_REG_STEP
#define GPIB_REG_INT_STAT1     1*GPIB_REG_STEP
#define GPIB_REG_INT_STAT2     2*GPIB_REG_STEP
#define GPIB_REG_SER_POLL_STAT 3*GPIB_REG_STEP
#define GPIB_REG_ADR_STAT      4*GPIB_REG_STEP
#define GPIB_REG_CMD_PASS_THRU 5*GPIB_REG_STEP
#define GPIB_REG_ADR0          6*GPIB_REG_STEP
#define GPIB_REG_ADR1          7*GPIB_REG_STEP

// Macro's for reading/writing to 7210 chip's registers 
// linux ones...
//#define GPIB_DEBUG
#ifdef DEBUG
#define GPIB_BYTE_OUT(Base,RegOffset,OutValue)  printf("%x\n",RegOffset);(outb(OutValue,(Base+RegOffset)))
#else
#define GPIB_BYTE_OUT(Base,RegOffset,OutValue)  (outb_p(OutValue,(Base+RegOffset)))
#endif
#define GPIB_BYTE_IN(Base,RegOffset) (inb_p(Base+RegOffset))   

#define GPIB_baseaddress int

// Prototypes for gpib_driver.c 
void GPIB_pause		(void);
int GPIB_set_EOT	(int); 
int GPIB_initboard 	(GPIB_baseaddress,int);
int GPIB_readdata 	(char *, int);
int GPIB_writedata 	(char *, int);
int GPIB_sendcommand 	(char);
int GPIB_standby 	(void);
int GPIB_active 	(void);
int GPIB_endhandshake 	(void);
int GPIB_getchipstatus 	(void);
int GPIB_init488chip 	(void);
int GPIB_IFC_strobe     (void);
int GPIB_RENLine        (int);
#endif
